local RezzMon = LibStub("AceAddon-3.0"):GetAddon("RezzMon")
local media = LibStub("LibSharedMedia-3.0")

local frameBuf = {
	["Frame"] = {},
	["StatusBar"] = {}
}

local function createFrame(frameType, frameParent)
	local frame = nil
	if #frameBuf[frameType] > 0 then
		frame = tremove(frameBuf[frameType])
		frame:SetParent(frameParent)
	else
		frame = CreateFrame(frameType, "", frameParent)
	end
	return frame
end

local function deleteFrame(frame)
	frame:SetScript("OnShow", nil)
	frame:SetScript("OnHide", nil)
	frame:SetScript("OnUpdate", nil)
	frame:Hide()
	tinsert(frameBuf[frame:GetObjectType()], frame)
end

local function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end


local BarTemplate = createFrame("Frame")
local BarTemplate_MT = {__index = BarTemplate}

function BarTemplate:AttachStopCallback(func)
	self.stopCallback = func
end

function BarTemplate:Start()
	self:ApplySettings()
	self.Text:SetText(self.label)
	self.Bar:SetMinMaxValues(0, self.duration)
	self.Bar:SetValue(0)
	self.remaining = self.duration
	self:Show()
end

function BarTemplate:Stop()
	self:Hide()
	if self.stopCallback then
		self:stopCallback(self)
	end
	deleteFrame(self.Bar)
	self.Bar = nil
	deleteFrame(self)
	self = nil
end

function BarTemplate:SetDuration(duration)
	self.duration = floor(duration)
end

function BarTemplate:SetRemaining(remaining)
	self.remaining = floor(remaining)
end

function BarTemplate:ApplySettings()
	self:SetHeight(RezzMon.db.profile.height)
	self:SetWidth(RezzMon.db.profile.width)
	
	local color = RAID_CLASS_COLORS[select(2, UnitClass(self.unit))]
	self.Bar:SetStatusBarTexture(media:Fetch("statusbar", RezzMon.db.profile.texture))
	self.Bar:SetStatusBarColor(color.r, color.g, color.b, 1.0)
	
	self.Text:SetFont(media:Fetch("font", RezzMon.db.profile.font), RezzMon.db.profile.fontSize)
	self.TimeText:SetFont(media:Fetch("font", RezzMon.db.profile.font), RezzMon.db.profile.fontSize)
end

local function onUpdate(self, timeSinceLast)
	self.remaining = self.remaining - floor(timeSinceLast*1000)
	local diff = self.duration - self.remaining
	if diff < -5000 then
		self:Stop()
	end
	self.Bar:SetValue(diff)
	self.TimeText:SetText(round(self.remaining/1000,1))
end

local function onShow(self)
	self:SetScript("OnUpdate", onUpdate)
end

local function onHide(self)
	self:SetScript("OnUpdate", nil)
end

RezzMon.BarTemplate = {}

function RezzMon.BarTemplate:New(unit, label)
	local bar = setmetatable(createFrame("Frame", UIParent), BarTemplate_MT)
	bar.unit = unit
	bar.label = label
	
	bar:SetFrameStrata("MEDIUM")
	bar:SetScript("OnShow", onShow)
	bar:SetScript("OnHide", onHide)
	bar:SetClampedToScreen(true)
	
	bar:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	                 tile = true, tileSize = 16, edgeSize = nil, edgeFile = nil,
	                 insets = {left = 0, right = 0, top = 0, bottom = 0} })
	bar:SetBackdropColor(0,0,0,0.8)
	
	bar.Bar = createFrame("StatusBar", bar)
	bar.Bar:ClearAllPoints()
	bar.Bar:SetAllPoints(bar)
	if not bar.Text then
		bar.Text = bar.Bar:CreateFontString(nil, "OVERLAY")
	end
	bar.Text:SetJustifyH("LEFT")
	bar.Text:SetJustifyV("MIDDLE")
	bar.Text:ClearAllPoints()
	bar.Text:SetPoint("LEFT", self.Bar, "LEFT", 2, 0)
	if not bar.TimeText then
		bar.TimeText = bar:CreateFontString(nil, "OVERLAY")
	end
	bar.TimeText:SetJustifyH("LEFT")
	bar.TimeText:SetJustifyV("MIDDLE")
	bat.TimeText:ClearAllPoints()
	bat.TimeText:SetPoint("LEFT", self.Bar, "RIGHT", 2, 0)
	
	bar.Bar:Show()
	bar:Hide()
	
	return bar
end